# FIND_BOTAN in this file is a copy of Monotone's MTN_FIND_BOTAN,
# copied with modifications from net.venge.monotone.stripped's
# file m4/botan.m4
#
# FIND_BOTAN is (C) 2008 markus@bluegap.ch
#
# Monotone (and thus presumably the m4 FIND_BOTAN macro) is
# distributed under the GNU GPL v2 or later
#
# I do not believe the inclusion or use of this macro implies that
# Ajisai as a whole is licensed under the GNU GPL. However I am not a
# lawyer (thank Goddess).

# Check for a function; if it doesn't exist, puke
AC_DEFUN([REQUIRE_FUNC], [
   AC_CHECK_FUNCS($1)
   if test $ac_cv_func_$1 = no; then
     AC_MSG_ERROR([Ajisai needs the function $1(), but it cannot be found]);
   fi
])


AC_DEFUN([FIND_BOTAN],
[
  AC_MSG_CHECKING([for Botan])
  if test -n "`type -p botan-config`"; then
    BOTAN_VERSION="`botan-config --version`"
    BOTAN_CPPFLAGS="`botan-config --cflags`"
    BOTAN_LIBS="`botan-config --libs`"

    found_botan=yes

    # make sure we have to do with a supported botan version
    save_CPPFLAGS="$CPPFLAGS"
    CPPFLAGS="$CPPFLAGS $BOTAN_CPPFLAGS"
    AC_PREPROC_IFELSE([
#include <botan/version.h>

#if BOTAN_VERSION_CODE < BOTAN_VERSION_CODE_FOR(1,7,23)
#error "Botan is too old"
#endif],
    [botan_version_match=yes],
    [botan_version_match=no])
    if test $botan_version_match = no; then
      AC_MSG_RESULT([no])
      AC_MSG_ERROR([Your botan library is too old ($BOTAN_VERSION).])
    fi

    # check against unknown versions from the future and warn
    AC_PREPROC_IFELSE([
#include <botan/version.h>

#if BOTAN_VERSION_CODE >= BOTAN_VERSION_CODE_FOR(1,9,0)
#error "Botan from the future"
#endif],
    [botan_version_match=yes],
    [botan_version_match=no])

    CPPFLAGS="$save_CPPFLAGS"
    AC_MSG_RESULT([yes (version $BOTAN_VERSION)])

    if test $botan_version_match = no; then
      AC_MSG_WARN([Your botan library version ($BOTAN_VERSION) is newer than expected. Ajisai might not build cleanly.])
    fi

    AC_MSG_NOTICE([using botan compile flags: "$BOTAN_CPPFLAGS"])
    AC_MSG_NOTICE([using botan link flags: "$BOTAN_LIBS"])

    AC_SUBST(BOTAN_LIBS)
    AC_SUBST(BOTAN_CPPFLAGS)
  else
    found_botan=no
    AC_MSG_RESULT([no])
    AC_MSG_ERROR([Botan cannot be found.])
  fi
])

