/**
* Exceptions Header File
* (C) 2004-2006 Jack Lloyd
*/

#ifndef AJISAI_EXCEPTION_H__
#define AJISAI_EXCEPTION_H__

#include <ajisai/magic.h>
#include <exception>
#include <string>

namespace Ajisai {

/**
* Exception Base Class
*/
class Exception : public std::exception
   {
   public:
      const char* what() const throw() { return msg.c_str(); }
      Alert_Type type() const throw() { return alert_type; }

      Exception(Alert_Type type,
                const std::string& err_msg = "Unknown error")
         {
         msg = "Ajisai: " + err_msg;
         alert_type = type;
         }
      virtual ~Exception() throw() {}
   private:
      std::string msg;
      Alert_Type alert_type;
   };

/**
* Invalid_Argument Exception
*/
struct Invalid_Argument : public Exception
   {
   Invalid_Argument(const std::string& err = "") :
      Exception(CLOSE_NOTIFY, err) {}
   };

/**
* Unexpected_Message Exception
*/
struct Unexpected_Message : public Exception
   {
   Unexpected_Message(const std::string& err) :
      Exception(UNEXPECTED_MESSAGE, err) {}
   };

/**
* Internal_Error Exception
*/
struct Internal_Error : public Exception
   {
   Internal_Error(const std::string& err) :
      Exception(INTERNAL_ERROR, err) {}
   };

/**
* Policy_Violation Exception
*/
struct Policy_Violation : public Exception
   {
   Policy_Violation(Alert_Type type, const std::string& err) :
      Exception(type, "Policy violation: " + err) {}
   };

/**
* Format_Error Exception
*/
struct Format_Error : public Exception
   {
   Format_Error(Alert_Type type, const std::string& str = "") :
      Exception(type, str) {}
   };

/**
* Encoding_Error Exception
*/
struct Encoding_Error : public Format_Error
   {
   Encoding_Error(Alert_Type type, const std::string& err) :
      Format_Error(type, "Encoding error: " + err) {}
   };

/**
* Decoding_Error Exception
*/
struct Decoding_Error : public Format_Error
   {
   Decoding_Error(const std::string& err) :
      Format_Error(DECODE_ERROR, "Decoding error: " + err) {}
   };

/**
* Stream_IO_Error Exception
*/
struct Stream_IO_Error : public Exception
   {
   Stream_IO_Error(const std::string& err) :
      Exception(INTERNAL_ERROR, "I/O error: " + err) {}
   };

/**
* Integrity Failure Exception
*/
struct Integrity_Failure : public Exception
   {
   Integrity_Failure(Alert_Type type, const std::string& err) :
      Exception(type, "Integrity failure: " + err) {}
   };

}

#endif
