/*************************************************
* TLS Session Key Header File                    *
* (C) 2004-2006 Jack Lloyd                       *
*************************************************/

#ifndef AJISAI_SESSION_KEYS_H__
#define AJISAI_SESSION_KEYS_H__

#include <ajisai/suites.h>
#include <botan/symkey.h>

namespace Ajisai {

/*************************************************
* TLS Session Keys                               *
*************************************************/
class SessionKeys
   {
   public:
      SymmetricKey client_cipher_key() const;
      SymmetricKey server_cipher_key() const;

      SymmetricKey client_mac_key() const;
      SymmetricKey server_mac_key() const;

      InitializationVector client_iv() const;
      InitializationVector server_iv() const;

      SecureVector<byte> master_secret() const;

      SessionKeys() {}
      SessionKeys(const CipherSuite&, Version_Code, const MemoryRegion<byte>&,
                  const MemoryRegion<byte>&, const MemoryRegion<byte>&);
   private:
      SymmetricKey ssl3_keygen(u32bit, const MemoryRegion<byte>&,
                               const MemoryRegion<byte>&,
                               const MemoryRegion<byte>&);
      SymmetricKey tls1_keygen(u32bit, const MemoryRegion<byte>&,
                               const MemoryRegion<byte>&,
                               const MemoryRegion<byte>&);

      SecureVector<byte> master_sec;
      SymmetricKey c_cipher, s_cipher, c_mac, s_mac;
      InitializationVector c_iv, s_iv;
   };

}

#endif
